////////////////////////////////////////////
//   Class ODES                           //
//                                        //
//   G. Voyatzis, 2012                    //
//   voyatzis@auth.gr
////////////////////////////////////////////
// This class integrates a system of Ordinary Differential Equations (ODES) using Bulirsch-Stoer Method
// 

#ifndef  _GV_ODES_March_2012_
#define  _GV_ODES_March_2012_

#include <iostream>
#include <cassert>
#include <cmath>

#ifndef _BULSTOERSPARAMETERS_
#define _BULSTOERSPARAMETERS_
#define IMAX 11 
#define SHRINK 0.95
#define GROW 1.2
#define NUSE 7
#endif

class ODES{
private:
    int     neq;                                             //number of ODES
    void    (*derivs)(double t, double y[], double dydt[]);    //pointer to function of ODES    
    
   double BSAcc; //accuracy of calculations
	double BSMinStep; //minimum time step 
	double BSMaxStep; //maximum time step
	int ner; //number of variables (starting from the 1st) to take into acount for error calculation
	int BSINITIALIZED; //bool true if BS is already initialized
	double *Bx; //used
	double **Bd;//in extrapolation (preserve values through loops)
	//pointers for internal use
	double *y, *ysav, *dysav, *yseq, *yerr, *ym, *ynn;

//Polynomial Extrapolation 
    void rzextr(int iest, double xest, double yest[], double yz[], double dy[]);
        //  xest
        //  yest
        //  yz
        //  dy

//Modified Midpoint Method (private,for use in Bulirsch-Stoer)
    void mmid(double y[], double dydx[], double t0, double dt0, int nstep, double yout[]);
        //  y      Current value of dependent variable (initial input data)
        //  dydx   derivative data produced by calling function "deriv"
        //  t0     Independent variable (usually time)
        //  dt0    total step to be made
        //  nstep  max number of substeps to be used    

public:
//Constructor
    ODES();            
        //  numODES                                     number of ODES
        //  DEs_system(double, nVector&, nVector&)      function of ODES    
 
//Constructor For BS only!!
    ODES(int numODEs, void ODEs_system(double , double [], double []), double minstep, double maxstep, double accuracy, int n_error_check);            
        //  numODEs                                     number of ODEs
        //  DEs_system(double, double[], double[])      function of ODEs  
        //  min/max step                                minimum/maximum step of integration 
        //  accuracy                                    accuracy of calculations
		//  n_error_check : number of (first) deqs used for accuary control
	

    ~ODES(void);                                                                //Destructor 
	
	
//Basic step Integration
    int bsstep(double y0[], double dydx[], double dt0, double yout[], double *dtnext);        
	    //  y0      Current value of dependent variable (initial input data)
        //  dydx    derivative data produced by calling function "deriv"
	    //  dt0     time step for integration (dt0 may shrinks internaly - Correct step done = yout[0]-y0[0] 
	    //  yout    output data
	    //  dtnext  suggested next step, (always smaller than MAXSTEP defined)

//Bulirsch-Stoer Interval Integration 
    int bsDTstep(double yin[], double DT, double *dt, double yout[]);   //Bulirsch-Stoer Integrator for time T  (from t0 to t0+T)
        //  yin     array of Initial conditions (yin[0]=t0)
        //  DT       Time for integration
        //  dt      internal time step (dt<=T)
        //  yout    array of Final conditions (yout[0]=t0+T)
        //  return  1: in case of complete integration
        //      or  0: in case of error integration
	void open(int numODES, void ODES_system(double , double y[], double dydt[]), double minstep, double maxstep, double acc, int n_error_check);
	int test(void);
};

//End of Class ODES



#endif